package com.laguerre.kan;

public class Behaviour extends Component {
	private static enum ObjectLifeState {
		DID_CREATE,
		DID_AWAKE,
		LIVING,
		SLEEPING,
		DESTROYING,
		DESTROYED
	}

	private ObjectLifeState state;
	private boolean enabled;

	public Behaviour() {
		this.state = ObjectLifeState.DID_CREATE;
		this.enabled = true;
	}

	public final boolean isEnabled() {
		return this.enabled;
	}

	public final void setEnabled(boolean enabled) {
		if (enabled != this.enabled) {
			if (enabled && this.state == ObjectLifeState.SLEEPING) {
				// re-continue
				this.state = ObjectLifeState.LIVING;
			}
			this.enabled = enabled;
		}
	}

	final void doLifeStep() {
		switch (this.state) {
		case DID_CREATE:
			if (this.enabled) {
				onAwake();
				this.state = ObjectLifeState.DID_AWAKE;
			}
			break;
		case DID_AWAKE: {
			if (this.enabled) {
				onEnable();
				onStart();
				this.state = ObjectLifeState.LIVING;
			}
			break;
		}
		case LIVING:
			if (this.enabled) {
				onUpdate();
			} else {
				onDisable();
				this.state = ObjectLifeState.SLEEPING;
			}
			break;
		case DESTROYING:
			if (this.enabled) {
				onDisable();
			}
			this.state = ObjectLifeState.DESTROYED;
			break;
		case DESTROYED:
			onDestroy();
			// Remove this behavior from the parent SceneObject.
			this.getSharedSOb().removeComponent(this);
			break;
		default:
			break;
		}
	}

	final void destroy() {
		if (this.state != ObjectLifeState.DESTROYED) {
			this.state = ObjectLifeState.DESTROYING;
		}
	}

	public void onAwake() {
	}

	public void onEnable() {
	}

	public void onDisable() {
	}

	public void onStart() {
	}

	public void onUpdate() {
	}

	public void onBeforeRenderingCameraPass(int passNum) {
	}

	public void onBeforeRenderingImageEffectsPass(int passNum) {
	}

	public void onDestroy() {
	}
}
